<?php

namespace Pinpoint\BlankProduct\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;

/**
 * Class Quantity
 * @package Pinpoint\BlankProduct\Helper
 */
class Quantity extends AbstractHelper
{
    /** @var \Magento\Catalog\Model\ProductFactory ProductFactory */
    protected $_productFactory;

    /**
     * @var \Magento\CatalogInventory\Model\Stock\Item
     */
    protected $stockItem;

    /**
     * @var \Magento\CatalogInventory\Model\StockRegistry
     */
    protected $stockRegistry;

    /**
     * Quantity constructor.
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \Magento\CatalogInventory\Model\Stock\Item $stockItem
     * @param \Magento\CatalogInventory\Model\StockRegistry $stockRegistry
     * @param Context $context
     */
    public function __construct(
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\CatalogInventory\Model\Stock\Item $stockItem,
        \Magento\CatalogInventory\Model\StockRegistry $stockRegistry,
        Context $context
    ) {
        parent::__construct($context);

        $this->stockItem = $stockItem;
        $this->_productFactory = $productFactory;
        $this->stockRegistry = $stockRegistry;
    }

    /**
     * @param $productId
     * @return bool
     */
    public function getBlankQuantityAvailable($productId)
    {
        $product = $this->_productFactory->create()->load($productId);
        $productStockItem = $this->stockItem->load($product->getId(), 'product_id');
        if ($productStockItem->getQty() > 0 && !$productStockItem->getIsInStock()) {
            return false;
        }

        if ($product->getCustomAttribute('blank_products')
            && trim($product->getCustomAttribute('blank_products')->getValue()) != '') {
            $blankProducts = $product->getCustomAttribute('blank_products')->getValue();
            $bang = explode(',', $blankProducts);

            foreach ($bang as $product) {
                $productData = explode("|", $product);

                /** @var \Magento\Catalog\Model\Product $blankProduct */
                $blankProduct = $this->_productFactory->create()->loadByAttribute('sku', $productData[0]);
                if (!$blankProduct) {
                    return false;
                }

                $stockItem = $this->stockItem->load($blankProduct->getId(), 'product_id');
                if ($stockItem->getQty() < $productData[1] && !$blankProduct->isSalable()) {
                    return false;
                }
            }

            return true;
        }

        if ($customOptions = $product->getOptions()) {
            foreach ($customOptions as $option) {
                $values = $option->getValues();
                $hasBlank = false;
                foreach ($values as $optionValue) {
                    if (preg_match('/(?<=\#)(.*?)(?=\#)/', $optionValue->getSku(), $blankSku)) {
                        $_blankProduct = $this->_productFactory->create()->loadByAttribute('sku', $blankSku[0]);
                        if ($_blankProduct && $this->stockRegistry->getStockItem($_blankProduct->getId())->getIsInStock()) {
                            $hasBlank = true;
                        }
                    }
                }

                if ($hasBlank) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * @param $productId
     * @return int
     */
    public function getMaxBlankQuantity($productId)
    {
        $qty = 0;
        if ($this->getBlankQuantityAvailable($productId)) {
            $product = $this->_productFactory->create()->load($productId);
            if ($product->getCustomAttribute('blank_products')
                && trim($product->getCustomAttribute('blank_products')->getValue()) != '') {
                $blankProducts = $product->getCustomAttribute('blank_products')->getValue();
                $bang = explode(',', $blankProducts);
                $products = [];
                $blankProductData = [];

                foreach ($bang as $product) {
                    $productData = explode("|", $product);
                    $blankProductData[] = $productData;

                    /** @var \Magento\Catalog\Model\Product $blankProduct */
                    $blankProduct = $this->_productFactory->create()->loadByAttribute('sku', $productData[0]);
                    $stockItem = $this->stockItem->load($blankProduct->getId(), 'product_id');

                    $products[$blankProduct->getSku()] = [
                        'product' => $blankProduct,
                        'remaining_qty' => $stockItem->getQty()
                    ];
                }

                $foundTotalQuantity = false;
                while (!$foundTotalQuantity) {
                    foreach ($products as &$product) {
                        $requiredQty = $this->getRequiredQty($product['product']->getSku(), $blankProductData);
                        if ($product['remaining_qty'] >= $requiredQty) {
                            $product['remaining_qty'] -= $requiredQty;
                        } else {
                            $foundTotalQuantity = true;
                        }
                    }

                    if (!$foundTotalQuantity) {
                        $qty++;
                    }
                }
            }
        }

        return $qty;
    }

    /**
     * @param $productId
     * @param bool $returnItem
     * @return bool|int|\Magento\CatalogInventory\Api\Data\StockItemInterface
     */
    public function getBlankProductStockItem($productId, $returnItem = false)
    {
        $_product = $this->_productFactory->create()->load($productId);
        if ($_product->getCustomAttribute('blank_products') &&
            trim($_product->getCustomAttribute('blank_products')->getValue()) != '') {
            // get stock for blank product if it exists
            if ($this->getBlankQuantityAvailable($productId)) {
                $products = explode(',', $_product->getCustomAttribute('blank_products')->getValue());
                $_blankProduct = $this->_productFactory->create()->loadByAttribute('sku', explode("|", $products[0])[0]);
                if ($returnItem) {
                    return $this->stockRegistry->getStockItem($_blankProduct->getId());
                } else {
                    return $this->stockRegistry->getStockItem($_blankProduct->getId())->getIsInStock();
                }
            }
        }

        if ($customOptions = $_product->getOptions()) {
            foreach ($customOptions as $option) {
                $values = $option->getValues();
                $hasBlank = false;
                foreach ($values as $optionValue) {
                    if (preg_match('/(?<=\#)(.*?)(?=\#)/', $optionValue->getSku(), $blankSku)) {
                        $_blankProduct = $this->_productFactory->create()->loadByAttribute('sku', $blankSku[0]);
                        if ($_blankProduct && $this->stockRegistry->getStockItem($_blankProduct->getId())->getIsInStock()) {
                            $hasBlank = true;
                            break;
                        }
                    }
                }

                if ($returnItem && $_blankProduct) {
                    return $this->stockRegistry->getStockItem($_blankProduct->getId());
                } else {
                    if ($hasBlank && $_blankProduct) {
                        return $this->stockRegistry->getStockItem($_blankProduct->getId())->getIsInStock();
                    }
                }
            }
        }

        return false;
    }

    public function getStockStatus($sku) {
        $product = $this->_productFactory->create()->loadByAttribute('sku', $sku);
        if ($product) {
            return $this->stockRegistry->getStockItem($product->getId())->getIsInStock();
        } else {
            return false;
        }
    }

    /**
     * @param $sku
     * @param $blankProducts
     * @return null
     */
    protected function getRequiredQty($sku, $blankProducts)
    {
        foreach ($blankProducts as $blankProduct) {
            if ($sku == $blankProduct[0]) {
                return $blankProduct[1];
            }
        }

        return null;
    }
}